"""
ANTLR4 generated grammar files.

To regenerate the parser, run:
    cd src/gsql2rsql/parser/grammar
    antlr4 -Dlanguage=Python3 -visitor Cypher.g4

This will generate:
    - CypherLexer.py
    - CypherParser.py
    - CypherVisitor.py
    - CypherListener.py
"""

# Placeholder for generated files
# The actual files need to be generated using ANTLR4

__all__: list[str] = []
